/* gpdnl.c
   Convert and download Motorola S19 file to GP20 POD
   Rev 1.4   new method of calculating twaits
   In GP20 POD, User EMRAM RAM starts at $100, instead of $50 or $A0

Copyright 1999, 2001  Jed Margolin (comments@jmargolin.com)

   1. Permission is granted to individuals to make copies 
      for their own use.

      All other use requires permission of the copyright holder.

   2. The author is not held liable for the use of the information
      contained in the article and accompanying documentation.

   3. The article and accompanying documentation may be freely 
      distributed as long as:

      a. Jed Margolin is given credit as the author.

      b. The article and/or accompanying documentation may not be 
         changed in any way without the copyright holder's consent.

      c. No money is charged for distributing the article and/or 
         accompanying documentation.

      d. All those receiving or distributing the article and/or
         accompanying documentation agree to be bound by these 
         terms (1 - 3). 
*/

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <process.h>

#define READRAM  0xB000
#define FALSE (0) 
#define	TRUE  (1) 
#define NUMLINES 25
#define NUMCOLS 80
#define RETURN 13  
#define UPARROW 72 
#define DOWNARROW 80
#define ESC 27

FILE *ifp;
extern int RBR,LSR;
extern int rcvbr(void);

long twait,twait100us,twait200us,twait1ms,twait10ms,twait1s;
char ifname[100],rootname[100],dfname[100];
char storeit[4096];
char path[50];
unsigned char line[130];
char portnum;

void wait1ms(void),wait100us(void),wait200us(void),wait1s(void);
void read_back(int),test1(void),test2(void),test3(void);

void interrupt isr_break(void);
void interrupt isr_tics(void);
void interrupt (*oldbreak)(void);
int tick_flag, break_flag;

int main(int argc, char *argv[], char *env[])
{
   long m,n,i;
   int numchars;
   char *p;
   unsigned char temp1,temp2,ver1,ver2;
//------------------------------------------------------------------------- 
// clrscr(); 
   printf("\n");
   printf("pNambic Systems GP20 POD Download Program         Rev 1.4  \n");
   printf("Copyright 1994,1997,1998,2001  Jed Margolin  All Rights Reserved\n\n");

/* printf("The value of argc is %d \n",argc);
   printf("These are the command-line arguments passed to main:\n");
   for(i=0; i<argc; i++)  printf("  argv[%d]: %s \n",i,argv[i]);     */

/* printf("\nThe environment string(s) on this system are:\n");
   for(i=0; env[i] != NULL; i++)  printf("   env[%d]: %s\n",i,env[i]);*/ 

// printf("\nFile is %s\n",argv[0]);   	

   strcpy(path,argv[0]); 
   numchars = strlen(path);
   for (n=numchars; n>0; n--)
   {
	if(path[n] == '\\')   break;        // 92
	else path[n] = 0;
   }
//   printf("\nPath is %s\n",path);	
//-------------------------------------------------------------------------
// Let's figure out these arguments 
   if((argc < 2) || (argc > 3))
   { 
      printf("Usage:\n");
      printf("  gpdnl <input filename> [port] ");
      printf("example: gpdnl ptst.s19 2\n");
      exit(1);
   }

   if(argc == 3)	{ portnum = atoi(argv[2]); }
   else			{ portnum = 2; }		// default
// ---------------------------------------------------------------------
// Test the processor speed and calculate the Serial Timeout value
// Save the old Timer Interrupt and put in our own 
   tick_flag = 0;
   twait = 0;
   oldbreak = getvect(0x1c);
//   printf("Old Timer Interrupt 1c: %8lX\n\n",oldbreak);

   setvect(0x1c,isr_tics) ;	// New Interrupt for Tic Interrupt 

   while(!tick_flag);
   tick_flag = 0;

   while(!tick_flag)   { twait++; }
   tick_flag = 0;

   setvect(0x1c,oldbreak);		// Restore the old Timer Interrupt
   twait200us = twait/1100;
   twait100us = twait/550;
   twait1ms = twait/55;
   twait10ms = twait1ms*10;
   twait1s = twait * 18;

/*
   printf("twait = 0x%lX \n",twait);
   printf("twait100us = 0x%lX \n",twait100us);
   printf("twait200us = 0x%lX \n",twait200us);
   printf("twait1ms = 0x%lX \n",twait1ms);
   printf("twait1s = 0x%lX \n",twait1s);
*/
//------------------------------------------------------------------------- 
// Save the old Keyboard Break Interrupt and put in our own
   break_flag = 0;
   oldbreak = getvect(0x1b);

/* gotoxy(1,23);
   printf("Old Break Interrupt 1b (Keyboard): %8lX\n",oldbreak);  */
   setvect(0x1b,isr_break) ;	/* New Interrupt for Keyboard Break  */
// ------------------------------------------------------------------------
// Input File
   strcpy(ifname,argv[1]);

   p = strchr(ifname,'.');		//  0 if no '.' 
   if(p == 0)   strcat(ifname,".s19");
// ------------------------------------------------------------------------
// Open the File
   printf("Input File: %s \n",ifname);

   if((ifp = fopen(ifname,"r")) == NULL)
   {
      printf("Can't open input file %s\n",ifname);
      exit(1);
   }
// ------------------------------------------------------------------------ 
// Initialize Serial Port
   serial_init();

// Test if it works
   rcvn();			// dummy read
   wait1ms();
   send_char(0);		// Are you there?

//   for(n=0; n<1024; n++)     { storeit[n] = rcvn(); }

   temp1 = rcvbr();
   temp2 = rcvbr();

   if( (temp1 == 0) && (temp2 == 0xAA) ) { printf("Emulator Ready\n"); }
   else
   {
      printf("Emulator Does Not Respond %02X %02X\n",temp1,temp2);
      fclose(ifp);
      setvect(0x1b,oldbreak);	   // Restore the old Keyboard Break Interrupt
      exit(1);
   }

// Get Boot ROM Version
   rcvn();			// dummy read 
   wait1ms();
   send_char(3);

// for(n=0;n<1024;n++)  storeit[n] = rcvn();
   temp1 = rcv();
   ver1 = rcv();
   ver2 = rcv();
   printf("P6POD ROM Version: %2X.%2X  \n",ver1,ver2 );
// ------------------------------------------------------------------------ 
// Do it 
   download();
// ------------------------------------------------------------------------ 
   fclose(ifp);
// ------------------------------------------------------------------------
   read_back(READRAM);
// ------------------------------------------------------------------------
// start the program
   rcvn();			// dummy read
   wait1ms();
   send_char(2);
   temp1 = rcv();

   if(temp1 != 2)   printf("Comm Error %02X\n,temp1");
// ------------------------------------------------------------------------
// That's the news and we are outa here
   setvect(0x1b,oldbreak);	  // Restore the old Keyboard Break Interrupt
   return(0);
}
// ========================================================================
// ========================================================================
asconv(char val)
{
   if( (val >= 48) && (val <= 57)  )  return(val-48);
   if( (val >= 65) && (val <= 70)  )  return(val-65+10);
   if( (val >= 97) && (val <= 102) )  return(val-97+10);
   return(0);
}
// ========================================================================
// Download Motorola S Format
download()
{
   while(fgets(line,128,ifp))
   {	

/* Start character */
      if(line[0] == 'S')
      {
/* Record Type */
         switch (line[1])
         {
	    case '1':		// Motorola S1 Format: 16 bit address 
	      if(mot_s1())	  return(1);
	      break ;

	    case '2':		// Motorola S2 Format: 24 bit address 
	      if(mot_s2())   return(1);
	      break ;

	    case '3':		// Motorola S3 Format: 32 bit address
	      if(mot_s3())   return(1);
	      break ;
				
	    case '0':		// Header for S1 appears to be optional
	      break ;		

	    case '8':		// Header for S2 appears to be optional
	      break ;		

	    case '7':		// Header for S3 appears to be optional
	      break ;		

	    case '9':		// Termination Record
	      break ;		

/*          default:
	      printf("File Format Error: Invalid Start Character \n");	*/

         }   // end 'case'
      }	  // end 'if'
   }   // end 'while'

   return(0);
}
// ------------------------------------------------------------------------ 
// Motorola S1 Format: 16 bit address
mot_s1()
{
   int byte_count,n,sptr;    
   int address;			   // address is normally long
   unsigned int byte;
   unsigned char checksum;

// Byte Count
   byte_count = asconv(line[2])*16 + asconv(line[3]) - 3;     // data bytes
   checksum = byte_count + 3;

// Address
   sptr = 0;
   address = 0;
   for(n=4;n<8;n++)  { address = address * 16 + asconv(line[n]); }
   checksum = checksum + asconv(line[4]) * 16 + asconv(line[5]);
   checksum = checksum + asconv(line[6]) * 16 + asconv(line[7]);

   for(n=8;n<(8+byte_count*2);n=n+2)
   {
      byte = asconv(line[n])*16 + asconv(line[n+1]);
      storeit[sptr++] = byte;
      checksum = checksum + byte;
   }

   checksum = checksum + asconv(line[n])*16 + asconv(line[n+1]) + 1;

// Checksum 
   if(checksum != 0)
   {
      printf("File Checksum Error: %X\n",checksum);
      return(1);
   }

// Send it
   send_block(byte_count,address,0);
   if(break_flag)   	{ return(1); }

   return(0);
}
// ------------------------------------------------------------------------
// Motorola S2 Format: 24 bit address
mot_s2()
{
   int byte_count,n,sptr;       
   int address;			// address is normally long
   unsigned int byte;
   unsigned char checksum;

// Byte Count
   byte_count = asconv(line[2])*16 + asconv(line[3]) - 4;     // data bytes
   checksum = byte_count + 4;

// Address
   sptr = 0;
   address = 0;
   for(n=4;n<10;n++)	{ address = address * 16 + asconv(line[n]); }
   checksum = checksum + asconv(line[4]) * 16 + asconv(line[5]);
   checksum = checksum + asconv(line[6]) * 16 + asconv(line[7]);
   checksum = checksum + asconv(line[8]) * 16 + asconv(line[9]);

   for(n=10;n<(10+byte_count*2);n=n+2)
   {
      byte = asconv(line[n])*16 + asconv(line[n+1]);
      storeit[sptr++] = byte;
      checksum = checksum + byte;
   }

   checksum = checksum + asconv(line[n])*16 + asconv(line[n+1]) + 1;

// Checksum
   if(checksum != 0)
   {
      printf("File Checksum Error: %X\n",checksum);
      return(1);
   }

// Send it
   send_block(byte_count,address,0);
   if(break_flag)   	{ return(1); }

   return(0);
}
// ------------------------------------------------------------------------
// Motorola S2 Format: 32 bit address 
mot_s3()
{
   int byte_count,n,sptr;
   int address;				// address is normally long
   unsigned int byte;
   unsigned char checksum;

// Byte Count
   byte_count = asconv(line[2])*16 + asconv(line[3]) - 5;     // data bytes 
   checksum = byte_count + 5;

// Address 
   sptr = 0;
   address = 0;
   for(n=4;n<12;n++)	{ address = address * 16 + asconv(line[n]); }
   checksum = checksum + asconv(line[4]) * 16 + asconv(line[5]);
   checksum = checksum + asconv(line[6]) * 16 + asconv(line[7]);
   checksum = checksum + asconv(line[8]) * 16 + asconv(line[9]);
   checksum = checksum + asconv(line[10]) * 16 + asconv(line[11]);

   for(n=12;n<(12+byte_count*2);n=n+2)
   {
      byte = asconv(line[n])*16 + asconv(line[n+1]);
      storeit[sptr++] = byte;
      checksum = checksum + byte;
   }

   checksum = checksum + asconv(line[n])*16 + asconv(line[n+1]) + 1;

// Checksum
   if(checksum != 0)
   {
      printf("File Checksum Error: %X\n",checksum);
      return(1);
   }	  

// Send it
   send_block(byte_count,address,0);
   if(break_flag)   	{ return(1); }

   return(0);
}
// ========================================================================
// ========================================================================
void read_back(int address)
{
   int i,n,j;

   for(n=0;n<8;n++)
   {
      rcvn();			// dummy read
      wait1ms();
      send_char(4);
      rcv();

      wait1ms();
      send_char(16);			/* get 16 bytes */
      wait200us();
      send_char( (address>>8)&0xFF);	// high address byte
      wait200us();
      send_char(address&0xFF);		// low address byte
//      for(i=0;i<1024;i++) storeit[i] = rcvn();    
      for(i=0;i<16;i++)	  storeit[i] = rcvbr();

      printf("%04X  ",address);
      for(i=0;i<16;i++)	  printf("%02X ",storeit[i]&0xFF );
      printf("\n");
      
      address = address + 16;
 
      if(kbhit())   break; 
   }

   printf("\n\n");
}
// ========================================================================
void test()
{
   int n;

   for(n=0;n<1024;n=n+3)
   {
      send_char(3);
      storeit[n] = rcv();
      storeit[n+1] = rcv();
      storeit[n+2] = rcv();
/*      printf("%02X %02X %02X \n",storeit[n]&0xFF,storeit[n+1]&0xFF,storeit[n+2]&0xFF);  */
   }
}
// ========================================================================
void test2()
{
   int n,temp;

   for(n=0;n<1024;n++)
   {
      send_char(7);
      storeit[n] = rcv();
      temp = storeit[n] ;
   }
}
// ========================================================================
down_test4()
{
   int i,n,address;

   address = 0;
   for(i=0;i<1;i++)
   {
      rcvn();			// dummy read 
      wait1ms();
      send_char(1);		// Send it
      rcv();

      wait200us();
      send_char(200);
      wait200us();
      send_char(0x02);
      wait200us();
      send_char(address);

      for(n=0;n<16;n++)
      {
         wait200us();
	 send_char(address);
	 address++;
      }
   }
   return(0);
}
// ========================================================================
down_test3()
{
   int i,n;

   rcvn();			// dummy read
   wait1ms();
   send_char(1);		// Send it
   rcv();

   wait200us();
   send_char(200);
   wait200us();
   send_char(0x02);
   wait200us();
   send_char(0x00);

   for(n=0;n<200;n++)
   {
      wait200us();
      send_char(n);
   }

   return(0);
}
// ========================================================================
down_test2()
{
   int i,n,byte;

   rcvn();			// dummy read
   wait1ms();
   send_char(1);		// Send it
   byte = rcv();

   wait200us();
   send_char(16);
   wait200us();
   send_char(0x02);
   wait200us();
   send_char(0x00);

   for(n=0;n<16;n++)
   {
      wait200us();
      send_char(n);
   }

   wait1ms();
   send_char(1);		/* Send it */
   byte = rcv();

   wait200us();
   send_char(16);
   wait200us();
   send_char(0x02);
   wait200us();
   send_char(0x10);

   for(n=0;n<16;n++)
   {
      wait200us();
      send_char(n+16);
   }

   return(0);
}
// ========================================================================
down_test()
{
   int n;

   for(n=0;n<1024;n++)		storeit[n] = n;

   for(n=0;n<16;n++)
   {
      send_block(16,0x0200+n*16,n*16);
   }

   return(0);
}
// ========================================================================
send_block(char number,int address,int index)
{
   char n,byte;

   wait1ms();
   send_char(1);		/* Send it */
   byte = rcv();

   wait200us();
   send_char(number);
   wait200us();
   send_char( (address>>8)&0xFF);
   wait200us();
   send_char( address&0xFF);

   for(n=0;n<number;n++)
   {
      wait200us();
      send_char(storeit[n+index]);
   }

   return(0);
}
// ========================================================================
// interrupt service routine from break key
void interrupt isr_break(void)
{
   break_flag = 1; 
}
// ------------------------------------------------------------------------
//interrupt service routine from timer
void interrupt isr_tics(void)
{
    tick_flag = 1;
}
// ========================================================================
// Wait 1 second
void wait1s()
{
   long m;

   for(m=0; m<twait1s; m++) ;
}
// ------------------------------------------------------------------------
// Wait n tics 
void wait_tics(int n)
{
   long m,m1,m2;

   n = n++;
   m1 = biostime(0,0);		// what time is it now?
   m2 = m1;
   while( abs(m2-m1) < n)	       { m2 = biostime(0,0); }
}
// ------------------------------------------------------------------------
void wait100us()
{
   long m;
   
   for(m=0; m<twait100us; m++);
}
// ------------------------------------------------------------------------ 
void wait200us()
{
   long m;
   
   for(m=0; m<twait200us; m++);
}
// ------------------------------------------------------------------------
void wait1ms()
{
   long m;
   
   for(m=0; m<twait1ms; m++);
}
// ------------------------------------------------------------------------
void wait10ms()
{
   long m;
   
   for(m=0; m<twait10ms; m++);
}
// ========================================================================
// ========================================================================
