/* (C) Copyright 1993  Jed Margolin  All Rights Reserved */
/* PSP1 GAL/EPROM Programmer */
/* serial.c */

#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <process.h>
#include <time.h>

/*#define PORTNUM 0x02		/* Serial Port */
#define BAUDRATE 19200		/* Serial Baud Rate */

#define DLAB 0x83		/* Divisor Latch Access Bit in LCR */
#define DTR 0x1			/* DTR bit in MCR */
#define RTS 0x2			/* RTS bit in MCR */
#define DR 0x1			/* Data ready bit in LSR */
#define THRE 0x20		/* Transmit Holding Register Empty in LSR */
#define DEFAULTBAUD 19200	/* Default baud rate */
#define XOFF ('S' & 0x1f)	/* Control-S is XOFF */

FILE *dfp;
extern char dfname[128];
extern char dspfile[20];
extern int portnum,break_flag;
extern void interrupt (*oldbreak)(void);

/* Serial Timeout: 1.2 secs, adjusted for processor speed */
extern long twait;

void send_word(int);		/* Send a Word */
void send_char(int);		/* Send a byte */
int send_go(int);		/* Send GO Command */
void xmit_lword(void);
int receive();
int rcvbr(void);
long buffer[8192];

struct baudtyp
	{
	unsigned baudrate;
	int divisor;
	};

struct baudtyp baudrates[4]=
	{
	   {38400,3},
	   {19200,6},
	   {9600,12}
	};

int divisor;			/* baud rate divisor for sio chip */
int portbase,RBR,THR,IER,IIR,LCR,MCR,LSR,MSR,DLL,DLH;
int block;
long totbytes;
int paddrarray[2]={0x3f8,0x2f8}; 

void serial_init()
{
	int display_fuse(int);
	int nadr,offset,i,j,k,remaining,addr,value,m,count,mask;
	long int n;
	int nchars,fuse,checksum;
	long *sptr ;
	char *fname,line[128],nline[128];
	int fuse_start,array_start,bytes;
/* ------------------------------------------------------------------------ */
/* portnum is an external that comes from main() */
	if((portnum < 1) || (portnum > 2))
	{
	    printf("%d is not a valid serial port number\n",portnum);
	    exit(1);
	}

	divisor = finddivisor(BAUDRATE);
	if(divisor == -1)
	{
	    printf("Invalid baud rate %s\n",BAUDRATE);
	    exit(1);
	}
	
	portbase = paddrarray[portnum-1]; /* get serial port base address */

/* set up pointers to registers */	
	RBR = portbase;
	THR = portbase;
	IER = portbase + 1;
	IIR = portbase + 2;
	LCR = portbase + 3;
	MCR = portbase + 4;
	LSR = portbase + 5;
	MSR = portbase + 6;
	DLL = portbase;
	DLH = portbase + 1;

/*	clrscr();
	printf("\nSerial Values\n");
	printf(" RBR: %x\n",inp(RBR));
	printf(" THR: %x\n",inp(THR));
	printf(" IER: %x\n",inp(IER));
	printf(" IIR: %x\n",inp(IIR));
	printf(" LCR: %x\n",inp(LCR));
	printf(" MCR: %x\n",inp(MCR));
	printf(" LSR: %x\n",inp(LSR));
	printf(" MSR: %x\n",inp(MSR));
	printf(" DLL: %x\n",inp(DLH));
	printf(" DLH: %x\n",inp(DLH));     */
	
	outp(LCR,DLAB);	  /* DLAB:Divisor,BREAK,Parity,Stop Bits,Wordlength */
/*	outp(LCR,DLAB | inp(LCR));	/* DLAB for divisor */
	
/*	oldbaud = inp(DLL) | (inp(DLH)<<8);	/* save old divisor */
	
	outp(DLL,divisor & 0xff);		/* set up new divisor */
	outp(DLH,divisor>>8);

	if((inp(DLL) | (inp(DLH)<<8)) != divisor)	/* check divisor */
	{	
		gotoxy(6,22) ;	/* XMENU */
		printf("comm port %d at address %x doesn't seem to be available\n",portnum,portbase);
		exit(1);
	}
	
	outp(LCR,(0x7f & inp(LCR)));	/* clear DLAB for transmit */
	outp(MCR,DTR+RTS);			/* raise DTR and RTS */
}
/* ======================================================================== */
int finddivisor(baudrate)
unsigned baudrate;
	{
	struct baudtyp *bp;
	
	for(bp=baudrates;bp->baudrate;++bp)
		if(bp->baudrate == baudrate)
			return(bp->divisor);
	return(-1);
	}

void send_word(dword)
int dword;
	{
	send_char(dword>>8);		/* high byte */
	send_char(dword&0xff);		/* low byte */
 	}
/* ----------------------------------------------------------------------- */
/* For Testing 
send_char(var)
char var;
	{
	printf(" %x ", var);
	}			*/
	
void send_char(c)
char c;
{
	int n;
	while(! (inp(LSR) & THRE));	/* wait for last char to be sent */
/*	xoff_check();		*/
	outp(THR,c);			/* send char */
}

void xoff_check()
{
	if(!(inp(LSR) & DR))		/* if no char waiting, can't be xoff*/
		return;
	return;
/*	while(inp(RBR) == XOFF)		 see if char is xoff char */
/*		while(!(inp(LSR) & DR));	 wait for next char */
}

int receive()
{
        int m;
	for(m=0;m<3000;m++) ;		/* wait for it to receive */
	if(!(inp(LSR) & DR))		/* if no char waiting, can't be xoff*/
		return(0);
	printf("%2x \n",inp(RBR));
	return(0);
}

int rcv()
{
	unsigned long m;

	for(m=0;m<twait;m++)
	{
	    if((inp(LSR) & DR))   break;
	}
	return(inp(RBR));
}

int rcvb()
{
        int m;
	while(!(inp(LSR) & DR))	    { if(kbhit())   break; }
	return(inp(RBR));
}

int rcvrdy()
{
	unsigned long m;

	for(m=0;m<twait;m++)
	{
	    if((inp(LSR) & DR))   return(1);
	}
	return(0);	/* we have timed out */
}

/* Wait a maximum of 1 second for a character */
/* Returns 0 if we have a character */
/* Returns 1 is we have timed out */
int waitchar()
{
        int status;
	time_t start,now;

	status = 0;
    	start = time(NULL);		/* what time is it now? */

    	while(!(inp(LSR) & DR))		/* if no char waiting, spin */
	{
	   now = time(NULL);

	   if(difftime(now,start) >= 2)  
	   {
	   	status = 1;
		break;
	   }

	}

	return(status);
}
/* ----------------------------------------------------------------------- */
/* Do a naked read */
rcvn()   
{
   return(inp(RBR));
}
/* ----------------------------------------------------------------------- */
parse_name(name,iname)
char *name,*iname;
{
    char *np;

    for(np=name;*np;np++)
    {
	if(*np == '.')
	{
	   strcpy(iname,name);
	   return(0);
	}
    }
    strcpy(iname,name);
    strcat(iname,".jed");
    return(0);
}
/* ----------------------------------------------------------------------- */
/* Do a dummy read */
void ser_read()    
{
   inp(RBR);
}
/* ----------------------------------------------------------------------- */
/* Do a dummy read */
void dummy_read()  
{
   inp(RBR);
}
/* ======================================================================== */
/* Receive serial byte or terminate program on keyboard break */  
int rcvbr()
{
   while(!(inp(LSR) & DR))		/* if no char waiting, spin */
   {
      if(break_flag)
      {
         setvect(0x1b,oldbreak); /* Restore the old Keyboard Break Interrupt*/
	 clrscr();
	 printf("Program Terminated by Keyboard Break\n");
         wait1s();
//	 clrscr();
	 exit(1);		 /* closes all files and exits */
      }
   }			
   return(inp(RBR));
}
/* ======================================================================== */
